#!/bin/sh

# This script is designed to create an ext3ci module based on the ext3
# module which is case-insensitive. Fortunately, the ext3 filesystem designer
# made this extremely easy -- just change the very first function in namei.c.
# The hard part is making a copy of the module so that you can select which
# filesystems to mount case-insensitive and which to mount normally. The nice
# part about using the seperate-module approach is that there is no
# performance penalty for normal filesystems.

# INSTRUCTIONS: 
# in the linux subtree, add a directory called fs/ext3ci. Copy
# this file into that directory and run it. 
# Reconfigure the kernel, adding ext3ci. Rebuild. Now, mount the desired ext3
# filesystem as type "ext3ci".


cp ../ext3/*.c .
cp ../../include/linux/ext3*.h .
cp ../ext3/Makefile .
rm *.o 2>>/dev/null
rm -f ../filesystems.a

H=`ls ext3*.h | grep -v ext3ci`
for i in $H ;do
  mv $i ext3ci${i##ext3}
done

for i in *.c *.h Makefile ;do
  sed '
	s/ext3/ext3ci/g
	s/ext3ci_group_desc/ext3_group_desc/g
	s/ext3ci_super_block/ext3_super_block/g
	s/<linux\/ext3ci_fs.h>/"ext3ci_fs.h"/g
	s/<linux\/ext3ci_jbd.h>/"ext3ci_jbd.h"/g
	s/ext3ci_debug/ext3_debug/g
	s/ext3ci_dir_entry/ext3_dir_entry/g
	s/ext3ci_sb/ext3_sb/g
	s/ext3ci_i\./ext3_i\./g
	# for ext3ci_fs.h
	s/u\.ext3ci_i/u.ext3_i/g
	# s/ext3ci_inode/ext3_inode/g
	s/ext3ci_test_bit/ext3_test_bit/g
	s/ext3ci_clear_bit/ext3_clear_bit/g
	s/ext3ci_set_bit/ext3_set_bit/g
	s/ext3ci_find_next_zero_bit/ext3_find_next_zero_bit/g
	s/ext3ci_find_first_zero_bit/ext3_find_first_zero_bit/g
	s/EXT3CI/EXT3/g
	s/CONFIG_EXT3_FS/CONFIG_EXT3CI_FS/g
	' ${i} > ${i}.2
  mv ${i}.2 ${i}
  echo ${i}
done

echo "Lowercase namei.c..."
LINE=`cat -n namei.c | grep memcmp | awk '{ print $1 }'`
BEFORE=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 1`
DATA=`tail +${LINE} namei.c | head -1`

head -${BEFORE} namei.c > namei.c.2
echo "/* ${DATA} */" >> namei.c.2
cat << EOM >> namei.c.2
  na = (unsigned char*) name;
  nb = (unsigned char*) de->name;
  for (; len>0; na++,nb++,len--)
    if (ext3ci_lowercasemap[(int) *na] != ext3ci_lowercasemap[(int) *nb])
	return 0;
  return 1;
EOM
tail +${AFTER} namei.c >> namei.c.2

LINE=`cat -n namei.c.2 | grep 'static.*int ext3ci_match' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} + 2`
AFTER=`expr ${LINE} + 3`

head -${INSERTAT} namei.c.2 > namei.c
echo "	unsigned char *na, *nb;" >> namei.c
tail +${AFTER} namei.c.2 >> namei.c

LINE=`cat -n namei.c | grep 'static.*int ext3ci_match' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} namei.c > namei.c.2
echo "extern const char *ext3ci_lowercasemap;" >> namei.c.2
tail +${AFTER} namei.c >> namei.c.2

mv namei.c.2 namei.c


echo "Lowercase namei.c"

LINE=`cat -n namei.c | grep 'static struct dentry .ext3ci_lookup' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} namei.c > namei.c.2
cat ext3ci-sed-2.4.data.1 >> namei.c.2
tail +${AFTER} namei.c >> namei.c.2

LINE=`cat -n namei.c.2 | grep 'bh = ext3ci_find_entry(dentry, &de);' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} namei.c.2 > namei.c
echo "	dentry->d_op = &ext3ci_dentry_operations;" >> namei.c
tail +${AFTER} namei.c.2 >> namei.c


echo "Lowercase super.c"

LINE=`cat -n super.c | grep 'struct super_block . ext3ci_read_super' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} super.c > super.c.2
echo "extern struct dentry_operations ext3ci_dentry_operations;" >> super.c.2
tail +${AFTER} super.c >> super.c.2

LINE=`cat -n super.c.2 | \
	grep 'ext3ci_setup_super (sb, es, sb->s_flags . MS_RDONLY);' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} + 0`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} super.c.2 > super.c
echo "	sb->s_root->d_op = &ext3ci_dentry_operations;" >> super.c
tail +${AFTER} super.c.2 >> super.c

echo "FIX ../Config.in"
# Fix fs/Config.in so that ext3ci can be included in the kernel
grep -v 'CONFIG_EXT3CI_FS' ../Config.in > ../Config.in.2
LINE=`cat -n ../Config.in.2 | grep 'tristate .* CONFIG_EXT3_FS' \
	| awk '{ print $1 }'`
INSERTAT=`expr ${LINE}`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} ../Config.in.2 > ../Config.in.3
echo "tristate 'Case Insensitive Ext3 fs support' CONFIG_EXT3CI_FS" >> ../Config.in.3
tail +${AFTER} ../Config.in.2 >> ../Config.in.3

mv ../Config.in.3 ../Config.in


echo "FIX ../Makefile"
# Fix fs/Makefile so that ext3ci can be included in the kernel
grep -v 'CONFIG_EXT3CI_FS' ../Makefile > ../Makefile.2
LINE=`cat -n ../Makefile.2 | grep 'CONFIG_EXT3_FS' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE}`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} ../Makefile.2 > ../Makefile.3
echo "subdir-\$(CONFIG_EXT3CI_FS)	+= ext3ci" >> ../Makefile.3
tail +${AFTER} ../Makefile.2 >> ../Makefile.3

mv ../Makefile.3 ../Makefile




